/* #############################
   #  Globals
   #############################
*/

html, body {
  background-color: #081966;
  color: white;
}

table {
  background-color: #263480;
}

input {
  border: 1px solid #081966;
        background: #DDDDFF;
}

select {
  border: 1px solid #081966;
        background: #DDDDFF;
}

a {
  color: #F5EE74;
}


/* #############################
   #  Tooltip style for hints
   #############################
*/

.hint-tip {
  color: black;
}

.hint-tip .hint-tip-top {
  background: transparent url(/img/rounded-box-orange-tl.png) no-repeat 0px 0px;
}

.hint-tip .hint-tip-top .hint-tip-c {
  background: transparent url(/img/rounded-box-orange-tr.png) no-repeat right 0px;
}

.hint-tip .hint-tip-bdy {
  background: transparent url(/img/rounded-box-orange-ml.png) repeat-y 0px 0px;
}

.hint-tip .hint-tip-bdy .hint-tip-c {
  background: transparent url(/img/rounded-box-orange-mr.png) repeat-y right 0px;
}

.hint-tip .hint-tip-bot {
  background: transparent url(/img/rounded-box-orange-bl.png) no-repeat 0px 0px;
}

.hint-tip .hint-tip-bot .hint-tip-c {
  background: transparent url(/img/rounded-box-orange-br.png) no-repeat right 0px;
}


/* #############################
   #  Tooltip style for EPG info
   #############################
*/

.info-win .info-win-top {
  background: transparent url(../img/info-win-t-l.png) no-repeat 0px 0px;
}

.info-win .info-win-top .info-win-c {
  background: transparent url(../img/info-win-t-r.png) no-repeat right 0px;
}

.info-win .info-win-top .info-win-c .info-win-t {
  color: #000;
}
.info-win .info-win-body {
  background: transparent url(../img/info-win-m-l.png) repeat-y 0px 0px;
}

.info-win .info-win-body .info-win-c {
  background: transparent url(../img/info-win-m-r.png) repeat-y right 0px;
}

.info-win .info-win-bot {
  background: transparent url(../img/info-win-b-l.png) no-repeat 0px 0px;
}

.info-win .info-win-bot .info-win-c {
  background: transparent url(../img/info-win-b-r.png) no-repeat right 0px;
}


/* #############################
   #  Menu
   #############################
*/

div.menu {
  background: #FCBC40  url(../img/menu_line_bg.png) repeat-x;
  border-top: 1px solid #FFFDDD;
  border-bottom: 1px solid #FFFDDD;
  color: #122DBA;
}

div.menu a {
  color: black;
}

a#login {
  color: red;
}

div.menu a.active {
  color: #122DBA;
}

div#pagemenu {
  background: #FFFFFF url(../img/bg_line.png) top repeat-x;
}

div#pagemenu div {
  background: #FFFFFF url(../img/bg_line_top.png) bottom repeat-x;
}

div#pagemenu div div {
  background: #122DBA;
  border-top: 1px solid #C0C0FF;
  border-bottom: 1px solid #C0C0FF;
}

div#pagemenu a {
  color: #FCBC40;
}

div#pagemenu a.active {
  color: #FFFDDD;
}

div#pagemenu span {
  color: #FCBC40;
}

div#pagemenu span.sep {
  color: #FCBC40;
}


/* #############################
   #  Info box (near logo)
   #############################
*/

div#infobox {
  border: 1px solid #FCBC40;
}

div#infobox div.st_header {
  background: #122DBA;
  border-bottom: 1px solid #C0C0FF;
}

div#infobox div.st_content {
  background: #081966 url(../img/bg_line_top.png) top left repeat-x;
}

div#infobox div.st_controls div.st_update {
        border-right: 1px solid  #FCBC40;
}


/* #############################
   #  General table-cell classes
   #############################
*/

table td.bottomrow {
  border-bottom: 1px solid #FFFDDD !important;
}

table td.leftcol {
  border-left: 1px solid #FFFDDD;
}

table td.rightcol {
  border-right: 1px solid #FFFDDD;
}


/* #############################
   #  MultiSchedule
   #############################
*/

table.mschedule tr td.even {
  background-color: #081999;
}

table.mschedule tr td.odd {
  background-color: #081966;
}

table.mschedule tr td.current_row {
  background-color: #FCB840;
}

table.mschedule a {
  color: inherit;
}


/* #############################
   #  Event
   #############################
*/

div.station div {
  background: url(../img/bg_box_l.png) top left no-repeat;
}

div.station div div {
  background: url(../img/bg_box_r.png) top right no-repeat;
}

div.station div div div {
  background: url(../img/bg_box_h.png) repeat-x;
}

div.station div div div a {
  color: #000000;
}

td div.station a {
  color: #FCBC40;
}

div.content {
  background: #263480 url(../img/bg_tools.png) top left repeat-y;
  border-left: 1px solid #FFFDDD;
  border-right: 1px solid #FFFDDD;
  border-bottom: 1px solid #FFFDDD;
}

div.__progress {
  border: 1px solid #C0C0FF;
}

div.__progress div.__elapsed {
  background-color: #FCB840;
}
img.channel_logo {
  background-color: #FCB84000;
}


/* #############################
   #  Timers
   #############################
*/

table.listing tr td {
  /* background: url(../img/bg_line.png) bottom repeat-x; */
        background: transparent;
  border-bottom: 1px solid #FCBC40;
}

table.listing tr.description td {
  background: #122DBA;
}

table.listing tr td.current {
  color: #FCBC40;
}

table.listing tr.spacer td {
  background-color: #081966;
}

table.listing a {
  color: #FCBC40;
}


/* #############################
   #  Blue background thingy
   #############################
*/

div.boxheader   {
  background: url(../img/bg_box_l.png) top left no-repeat;
}

div.boxheader div {
  background: url(../img/bg_box_r.png) top right no-repeat;
}

div.boxheader div div {
  background: url(../img/bg_box_h.png) repeat-x;
  color: black;
}


/* #############################
   #  Recordings
   #############################
*/

div.recordings {
  border: 1px solid #FFFDDD;
  background-color: #263480;
}

div.recording_item {
  /* background: url(../img/bg_line.png) bottom repeat-x; */
        background: transparent;
  border-bottom: 1px solid #FCBC40;
}


/* #############################
   #  Remote control keypad
   #############################
*/

div.screenshot {
  background-image: url(../img/tv.jpg);
}


/* #############################
   #  Dotted frame
   #############################
*/

div.dotted {
  border: 1px dotted #bbbbbb;
  padding: 3px;
  margin: 2px;
  float: left;
  background-color: #122dBA;
}


/* #############################
   #  Edit tables
   #############################
*/

table.formular tr td {
  /* background: url(../img/bg_line.png) bottom repeat-x; */
  background: transparent;
  border-bottom: 1px solid #FCBC40;
}

table.dependent {
  background-color: #263480;
}


/* #######################################
   #  InfoWin support styles for EPG boxes
   #######################################
*/

.info-win div.epg_content {
        background: transparent url(../img/bg_tools.png) top left repeat-y;
}


/* #############################
   # About box
   #############################
*/

.info-win div.epg_content div.about_head div div {
  background: #122DBA;
  border-top: 1px solid #C0C1DA;
  border-bottom: 1px solid #C0C1DA;
}
